@@ -34,7 +34,27 @@ from glances.history import GlancesHistory
 from glances.logger import logger
 from glances.events import glances_events
 from glances.thresholds import glances_thresholds
-from glances.timer import Counter
+from glances.timer import Counter, Timer
+
+
+fields_unit_short = {
+    'percent': '%'
+}
+
+fields_unit_type = {
+    'percent': 'float',
+    'percents': 'float',
+    'number': 'int',
+    'numbers': 'int',
+    'int': 'int',
+    'ints': 'int',
+    'float': 'float',
+    'floats': 'float',
+    'second': 'int',
+    'seconds': 'int',
+    'byte': 'int',
+    'bytes': 'int'
+}
 
 
 class GlancesPlugin(object):
@@ -44,7 +64,8 @@ class GlancesPlugin(object):
                  args=None,
                  config=None,
                  items_history_list=None,
-                 stats_init_value={}):
+                 stats_init_value={},
+                 fields_description=None):
         """Init the plugin of plugins class.
 
         All Glances' plugins should inherit from this class. Most of the
@@ -99,6 +120,17 @@ class GlancesPlugin(object):
         # Init the views
         self.views = dict()
 
+        # Hide stats if all the hide_zero_fields has never been != 0
+        # Default is False, always display stats
+        self.hide_zero = False
+        self.hide_zero_fields = []
+
+        # Set the initial refresh time to display stats the first time
+        self.refresh_timer = Timer(0)
+
+        # Init stats description
+        self.fields_description = fields_description
+
         # Init the stats
         self.stats_init_value = stats_init_value
         self.stats = None
@@ -131,20 +163,19 @@ class GlancesPlugin(object):
         """Return the key of the list."""
         return None
 
-    def is_enable(self, plugin_name=None):
+    def is_enabled(self, plugin_name=None):
         """Return true if plugin is enabled."""
         if not plugin_name:
             plugin_name = self.plugin_name
         try:
             d = getattr(self.args, 'disable_' + plugin_name)
         except AttributeError:
-            return True
-        else:
-            return d is False
+            d = getattr(self.args, 'enable_' + plugin_name, True)
+        return d is False
 
-    def is_disable(self, plugin_name=None):
+    def is_disabled(self, plugin_name=None):
         """Return true if plugin is disabled."""
-        return not self.is_enable(plugin_name=plugin_name)
+        return not self.is_enabled(plugin_name=plugin_name)
 
     def _json_dumps(self, d):
         """Return the object 'd' in a JSON format.
@@ -157,7 +188,9 @@ class GlancesPlugin(object):
             return json.dumps(d, ensure_ascii=False)
 
     def history_enable(self):
-        return self.args is not None and not self.args.disable_history and self.get_items_history_list() is not None
+        return self.args is not None and \
+            not self.args.disable_history and \
+            self.get_items_history_list() is not None
 
     def init_stats_history(self):
         """Init the stats history (dict of GlancesAttribute)."""
@@ -187,10 +220,10 @@ class GlancesPlugin(object):
                     # Stats is a list of data
                     # Iter throught it (for exemple, iter throught network
                     # interface)
-                    for l in self.get_export():
+                    for l_export in self.get_export():
                         self.stats_history.add(
-                            nativestr(l[item_name]) + '_' + nativestr(i['name']),
-                            l[i['name']],
+                            nativestr(l_export[item_name]) + '_' + nativestr(i['name']),
+                            l_export[i['name']],
                             description=i['description'],
                             history_max_size=self._limits['history_size'])
                 else:
@@ -301,10 +334,17 @@ class GlancesPlugin(object):
     def sorted_stats(self):
         """Get the stats sorted by an alias (if present) or key."""
         key = self.get_key()
-        return sorted(self.stats, key=lambda stat: tuple(map(
-            lambda part: int(part) if part.isdigit() else part.lower(),
-            re.split(r"(\d+|\D+)", self.has_alias(stat[key]) or stat[key])
-        )))
+        try:
+            return sorted(self.stats, key=lambda stat: tuple(map(
+                lambda part: int(part) if part.isdigit() else part.lower(),
+                re.split(r"(\d+|\D+)", self.has_alias(stat[key]) or stat[key])
+            )))
+        except TypeError:
+            # Correect "Starting an alias with a number causes a crash #1885"
+            return sorted(self.stats, key=lambda stat: tuple(map(
+                lambda part: part.lower(),
+                re.split(r"(\d+|\D+)", self.has_alias(stat[key]) or stat[key])
+            )))
 
     @short_system_name.setter
     def short_system_name(self, short_name):
@@ -413,7 +453,7 @@ class GlancesPlugin(object):
         if not isinstance(self.stats, list):
             return None
         else:
-            if value.isdigit():
+            if not isinstance(value, int) and value.isdigit():
                 value = int(value)
             try:
                 return self._json_dumps({value: [i for i in self.stats if i[item] == value]})
@@ -422,16 +462,61 @@ class GlancesPlugin(object):
                     "Cannot get item({})=value({}) ({})".format(item, value, e))
                 return None
 
+    def update_views_hidden(self):
+        """If the self.hide_zero is set then update the hidden field of the view
+        It will check if all fields values are already be different from 0
+        In this case, the hidden field is set to True
+
+        Note: This function should be called by plugin (in the update_views method)
+
+        Example (for network plugin):
+        __Init__
+            self.hide_zero_fields = ['rx', 'tx']
+        Update views
+            ...
+            self.update_views_hidden()
+        """
+        if not self.hide_zero:
+            return False
+        if (isinstance(self.get_raw(), list) and
+                self.get_raw() is not None and
+                self.get_key() is not None):
+            # Stats are stored in a list of dict (ex: NETWORK, FS...)
+            for i in self.get_raw():
+                if any([i[f] for f in self.hide_zero_fields]):
+                    for f in self.hide_zero_fields:
+                        self.views[i[self.get_key(
+                        )]][f]['_zero'] = self.views[i[self.get_key()]][f]['hidden']
+                for f in self.hide_zero_fields:
+                    self.views[i[self.get_key(
+                    )]][f]['hidden'] = self.views[i[self.get_key()]][f]['_zero'] and i[f] == 0
+        elif isinstance(self.get_raw(), dict) and self.get_raw() is not None:
+            #
+            # Warning: This code has never been tested because
+            # no plugin with dict instance use the hidden function...
+            #                       vvvv
+            #
+            # Stats are stored in a dict (ex: CPU, LOAD...)
+            for key in listkeys(self.get_raw()):
+                if any([self.get_raw()[f] for f in self.hide_zero_fields]):
+                    for f in self.hide_zero_fields:
+                        self.views[f]['_zero'] = self.views[f]['hidden']
+                for f in self.hide_zero_fields:
+                    self.views[f]['hidden'] = self.views['_zero'] and self.views[f] == 0
+        return True
+
     def update_views(self):
         """Update the stats views.
 
         The V of MVC
         A dict of dict with the needed information to display the stats.
         Example for the stat xxx:
-        'xxx': {'decoration': 'DEFAULT',
-                'optional': False,
-                'additional': False,
-                'splittable': False}
+        'xxx': {'decoration': 'DEFAULT',  >>> The decoration of the stats
+                'optional': False,        >>> Is the stat optional
+                'additional': False,      >>> Is the stat provide additional information
+                'splittable': False,      >>> Is the stat can be cut (like process lon name)
+                'hidden': False,          >>> Is the stats should be hidden in the UI
+                '_zero': True}            >>> For internal purpose only
         """
         ret = {}
 
@@ -440,12 +525,15 @@ class GlancesPlugin(object):
                 self.get_key() is not None):
             # Stats are stored in a list of dict (ex: NETWORK, FS...)
             for i in self.get_raw():
+                # i[self.get_key()] is the interface name (example for NETWORK)
                 ret[i[self.get_key()]] = {}
                 for key in listkeys(i):
                     value = {'decoration': 'DEFAULT',
                              'optional': False,
                              'additional': False,
-                             'splittable': False}
+                             'splittable': False,
+                             'hidden': False,
+                             '_zero': self.views[i[self.get_key()]][key]['_zero'] if i[self.get_key()] in self.views and key in self.views[i[self.get_key()]] and 'zero' in self.views[i[self.get_key()]][key] else True}
                     ret[i[self.get_key()]][key] = value
         elif isinstance(self.get_raw(), dict) and self.get_raw() is not None:
             # Stats are stored in a dict (ex: CPU, LOAD...)
@@ -453,7 +541,9 @@ class GlancesPlugin(object):
                 value = {'decoration': 'DEFAULT',
                          'optional': False,
                          'additional': False,
-                         'splittable': False}
+                         'splittable': False,
+                         'hidden': False,
+                         '_zero': self.views[key]['_zero'] if key in self.views and '_zero' in self.views[key] else True}
                 ret[key] = value
 
         self.views = ret
@@ -530,6 +620,32 @@ class GlancesPlugin(object):
         """Set the limits to input_limits."""
         self._limits = input_limits
 
+    def set_refresh(self, value):
+        """Set the plugin refresh rate"""
+        self.set_limits('refresh', value)
+
+    def get_refresh(self):
+        """Return the plugin refresh time"""
+        ret = self.get_limits(item='refresh')
+        if ret is None:
+            ret = self.args.time
+        return ret
+
+    def get_refresh_time(self):
+        """Return the plugin refresh time"""
+        return self.get_refresh()
+
+    def set_limits(self, item, value):
+        """Set the limits object."""
+        self._limits['{}_{}'.format(self.plugin_name, item)] = value
+
+    def get_limits(self, item=None):
+        """Return the limits object."""
+        if item is None:
+            return self._limits
+        else:
+            return self._limits.get('{}_{}'.format(self.plugin_name, item), None)
+
     def get_stats_action(self):
         """Return stats for the action.
 
@@ -680,6 +796,13 @@ class GlancesPlugin(object):
                               action_key=action_key,
                               log=True)
 
+    def is_limit(self, criticity, stat_name=""):
+        """Return true if the criticity limit exist for the given stat_name"""
+        if stat_name == "":
+            return self.plugin_name + '_' + criticity in self._limits
+        else:
+            return stat_name + '_' + criticity in self._limits
+
     def get_limit(self, criticity, stat_name=""):
         """Return the limit value for the alert."""
         # Get the limit for stat + header
@@ -752,6 +875,21 @@ class GlancesPlugin(object):
         except KeyError:
             return default
 
+    def is_show(self, value, header=""):
+        """Return True if the value is in the show configuration list.
+        If the show value is empty, return True (show by default)
+
+        The show configuration list is defined in the glances.conf file.
+        It is a comma separed list of regexp.
+        Example for diskio:
+        show=sda.*
+        """
+        # @TODO: possible optimisation: create a re.compile list
+        if self.get_conf_value('show', header=header) == []:
+            return True
+        else:
+            return any(j for j in [re.match(i, value) for i in self.get_conf_value('show', header=header)])
+
     def is_hide(self, value, header=""):
         """Return True if the value is in the hide configuration list.
 
@@ -760,8 +898,8 @@ class GlancesPlugin(object):
         Example for diskio:
         hide=sda2,sda5,loop.*
         """
-        # TODO: possible optimisation: create a re.compile list
-        return not all(j is None for j in [re.match(i, value.lower()) for i in self.get_conf_value('hide', header=header)])
+        # @TODO: possible optimisation: create a re.compile list
+        return any(j for j in [re.match(i, value) for i in self.get_conf_value('hide', header=header)])
 
     def has_alias(self, header):
         """Return the alias name for the relative header or None if nonexist."""
@@ -831,12 +969,96 @@ class GlancesPlugin(object):
             additional: True if the stat is additional (display only if space is available after optional)
             spittable: Line can be splitted to fit on the screen (default is not)
         """
-        return {'msg': msg, 'decoration': decoration, 'optional': optional, 'additional': additional, 'splittable': splittable}
+        return {'msg': msg,
+                'decoration': decoration,
+                'optional': optional,
+                'additional': additional,
+                'splittable': splittable}
 
     def curse_new_line(self):
         """Go to a new line."""
         return self.curse_add_line('\n')
 
+    def curse_add_stat(self, key, width=None,
+                       header='', separator='', trailer=''):
+        """Return a list of dict messages with the 'key: value' result
+
+          <=== width ===>
+        __key     : 80.5%__
+        | |       | |    |_ trailer
+        | |       | |_ self.stats[key]
+        | |       |_ separator
+        | |_ key
+        |_ trailer
+
+        Instead of:
+            msg = '  {:8}'.format('idle:')
+            ret.append(self.curse_add_line(msg, optional=self.get_views(key='idle', option='optional')))
+            msg = '{:5.1f}%'.format(self.stats['idle'])
+            ret.append(self.curse_add_line(msg, optional=self.get_views(key='idle', option='optional')))
+
+        Use:
+            ret.extend(self.curse_add_stat('idle', width=15, header='  '))
+
+        """
+        if key not in self.stats:
+            return []
+
+        # Check if a shortname is defined
+        if 'short_name' in self.fields_description[key]:
+            key_name = self.fields_description[key]['short_name']
+        else:
+            key_name = key
+
+        # Check if unit is defined and get the short unit char in the unit_sort dict
+        if 'unit' in self.fields_description[key] and self.fields_description[key]['unit'] in fields_unit_short:
+            # Get the shortname
+            unit_short = fields_unit_short[self.fields_description[key]['unit']]
+        else:
+            unit_short = ''
+
+        # Check if unit is defined and get the unit type unit_type dict
+        if 'unit' in self.fields_description[key] and self.fields_description[key]['unit'] in fields_unit_type:
+            # Get the shortname
+            unit_type = fields_unit_type[self.fields_description[key]['unit']]
+        else:
+            unit_type = 'float'
+
+        # Is it a rate ? Yes, compute it thanks to the time_since_update key
+        if 'rate' in self.fields_description[key] and self.fields_description[key]['rate'] is True:
+            value = self.stats[key] // self.stats['time_since_update']
+        else:
+            value = self.stats[key]
+
+        if width is None:
+            msg_item = header + '{}'.format(key_name) + separator
+            if unit_type == 'float':
+                msg_value = '{:.1f}{}'.format(value, unit_short) + trailer
+            elif 'min_symbol' in self.fields_description[key]:
+                msg_value = '{}{}'.format(self.auto_unit(int(value),
+                                                         min_symbol=self.fields_description[key]['min_symbol']),
+                                          unit_short) + trailer
+            else:
+                msg_value = '{}{}'.format(int(value), unit_short) + trailer
+        else:
+            # Define the size of the message
+            # item will be on the left
+            # value will be on the right
+            msg_item = header + '{:{width}}'.format(key_name, width=width-7) + separator
+            if unit_type == 'float':
+                msg_value = '{:5.1f}{}'.format(value, unit_short) + trailer
+            elif 'min_symbol' in self.fields_description[key]:
+                msg_value = '{:>5}{}'.format(self.auto_unit(int(value),
+                                                            min_symbol=self.fields_description[key]['min_symbol']),
+                                             unit_short) + trailer
+            else:
+                msg_value = '{:>5}{}'.format(int(value), unit_short) + trailer
+        decoration = self.get_views(key=key, option='decoration')
+        optional = self.get_views(key=key, option='optional')
+
+        return [self.curse_add_line(msg_item, optional=optional),
+                self.curse_add_line(msg_value, decoration=decoration, optional=optional)]
+
     @property
     def align(self):
         """Get the curse align."""
@@ -917,11 +1139,22 @@ class GlancesPlugin(object):
         return ret
 
     def _check_decorator(fct):
-        """Check if the plugin is enabled."""
+        """Check decorator for update method.
+        It checks:
+        - if the plugin is enabled.
+        - if the refresh_timer is finished
+        """
+
         def wrapper(self, *args, **kw):
-            if self.is_enable():
+            if self.is_enabled() and (self.refresh_timer.finished() or self.stats == self.get_init_value):
+                # Run the method
                 ret = fct(self, *args, **kw)
+                # Reset the timer
+                self.refresh_timer.set(self.get_refresh())
+                self.refresh_timer.reset()
             else:
+                # No need to call the method
+                # Return the last result available
                 ret = self.stats
             return ret
         return wrapper
